'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = _interopDefault(require('fs'));
var inky = require('inky');
var promisify = _interopDefault(require('util-promisify'));
var Handlebars = _interopDefault(require('handlebars'));
var juice = _interopDefault(require('juice'));

const inky$1 = new inky.Inky();
const readFile = promisify(fs.readFile);
readFile.sync = fs.readFileSync;

const coerceArray = value => value === undefined ? [] : Array.isArray(value) ? value : [value];

const compose = (...fns) => arg => fns.reduceRight((arg, fn) => fn(arg), arg);

const eachObj = (fn, obj) => Object.keys(obj).forEach(key => fn(key, obj[key], obj));

const inkify = html => inky$1.releaseTheKraken(html, { decodeEntities: false }).replace(/\{\{&gt;/g, '{{>').replace(/\{\{#&gt;/g, '{{#>').replace(/&apos;/g, '\'');

const registerPartials = (hbs, partials) => eachObj((name, partial) => hbs.registerPartial(name, inkify(partial)), partials);

const registerHelpers = (hbs, helpers) => eachObj((name, helper) => hbs.registerHelper(name, helper), helpers);

const FOUNDATION = readFile.sync(require.resolve('foundation-emails/dist/foundation-emails.css'), 'utf-8');
const BASE_TEMPLATE = readFile.sync(require.resolve('./base-template.hbs'), 'utf-8');

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/**
 * Creates a template compiler
 * @param {CompilerParams} globalParams
 */
function Compiler(globalParams = {}) {
  const hbs = Handlebars.create();

  const globalCss = [FOUNDATION, ...coerceArray(globalParams.css)];
  const BaseTemplate = hbs.compile(globalParams.baseTemplate || BASE_TEMPLATE);

  var _globalParams$helpers = globalParams.helpers;
  const helpers = _globalParams$helpers === undefined ? {} : _globalParams$helpers;
  var _globalParams$partial = globalParams.partials;
  const partials = _globalParams$partial === undefined ? {} : _globalParams$partial;

  registerHelpers(hbs, helpers);
  registerPartials(hbs, partials);

  /**
   * Compiles template
   * @param {CompilerInstanceParams} params
   */
  function compile(params) {
    const css = [...globalCss, ...coerceArray(params.css)];
    const template = hbs.compile(BaseTemplate({ body: params.template, css }));

    return compose(juice, inkify, data => template(data), data => globalParams.data ? _extends({}, globalParams.data, data) : data);
  }

  return compile;
}

/**
 * @typedef {object} CompilerParams
 * @property {object} data global template data
 * @property {object} partials handlebars partials
 * @property {object} helpers handlebars helpers
 * @property {string} baseTemplate base template
 * @property {string|string[]} css global style string(s)
 */

/**
 * @typedef {object} CompilerInstanceParams
 * @property {string} template template string
 * @property {string|string[]} css local style string(s)
 */

module.exports = Compiler;
